﻿using UnityEngine;
using System.Collections;

public class Hunter : MonoBehaviour
{
    private const float SHOOT_TIME = 0.8f;
    private const float SIGHT_DISTANCE = 3.0f;
    private const float SIGHT_WIDTH = 0.4f;

    private float _timeBeforeShoot;
    private bool _aiming;
    private bool _moving;

    public SpriteRenderer sprite;
    public LayerMask targetLayer;
    public Sprite normalSprite;
    public Sprite aimSprite;
    public AudioClip shotSound;
    public GameObject warningFeedback;
    public GameObject visionFeedback;

    public float moveSpeed = 1f;
    private Vector2 _moveStart;
    private Vector2 _moveTarget;
    private float _moveDistance;
    private float _moveProgress;

    private float _recoilAnim;

    private Vector2 _shootOrigin; // Precalculated from the sprite because it changes

	void Start ()
    {
        ResetTimeBeforeShoot();
        gameObject.AddComponent<AudioSource>();

        Bounds b = sprite.bounds;
        _shootOrigin = new Vector2(0, (b.size / 2f).y); // The pivot is centered
	}

    void ResetTimeBeforeShoot()
    {
        _timeBeforeShoot = SHOOT_TIME;
    }
	
	void Update ()
    {
        if (_moving)
        {
            UpdateMoving();
        }
        else
        {
            UpdateStanding();
        }
    }

    private void ShowWarning()
    {
        warningFeedback.SetActive(true);
    }

    private void MoveTo(Vector2 pos)
    {
        _moving = true;
        _aiming = false;
        //transform.localRotation = Quaternion.identity;
        _moveStart = transform.position;
        _moveTarget = pos;
        _moveDistance = Vector2.Distance(_moveStart, _moveTarget);
        _moveProgress = 0f;
        SetFlipped(_moveTarget.x < _moveStart.x);
    }

    private void UpdateMoving()
    {
        _moveProgress += Time.deltaTime * moveSpeed;
        if(_moveProgress >= _moveDistance)
        {
            _moving = false;
        }

        Vector2 pos = Vector2.Lerp(_moveStart, _moveTarget, _moveProgress / _moveDistance);
        transform.position = pos;

        // Oh! FOOD AHEAD
        Bunny bunny = LookForPrey();
        if(bunny != null)
        {
            ShowWarning();
            _moving = false;
            _timeBeforeShoot = 0; // To aim quickly
        }
    }

    Vector2 GetCurrentShootOrigin()
    {
        Vector2 v = transform.right;
        Vector3 pos = transform.position;
        return new Vector2(pos.x, pos.y) + _shootOrigin + 0.3f * v;
    }

    Bunny LookForPrey()
    {
        float sightDistance = SIGHT_DISTANCE;
        Vector2 cso = GetCurrentShootOrigin();
        RaycastHit2D hit = Physics2D.CircleCast(cso, SIGHT_WIDTH, transform.right, sightDistance, targetLayer);

        if(hit.collider != null)
        {
            return hit.collider.GetComponent<Bunny>();
        }
        else
        {
            return null;
        }
    }

    private void UpdateStanding()
    {
#if UNITY_EDITOR
        {
            //Vector3 shootOrigin3D = _shootOrigin;
            //Debug.DrawLine(transform.position + shootOrigin3D, transform.position + shootOrigin3D + Vector3.up);

            Vector3 shootOrigin = GetCurrentShootOrigin();
            Vector3 hOffset = new Vector3(0, SIGHT_WIDTH, 0);
            Vector3 v = transform.right;
            Debug.DrawLine(shootOrigin - hOffset, shootOrigin + v * SIGHT_DISTANCE - hOffset);
            Debug.DrawLine(shootOrigin + hOffset, shootOrigin + v * SIGHT_DISTANCE + hOffset);
        }
#endif
        if (!IsInField())
        {
            MoveToAnotherPlace();
            return;
        }

        _timeBeforeShoot -= Time.deltaTime;
        if (_timeBeforeShoot < 0f)
        {
            ResetTimeBeforeShoot();

            Bunny bunny = LookForPrey();
            //Debug.Log("Bunny: " + bunny);

            if (bunny != null)
            {
                if (!_aiming)
                {
                    _aiming = true;
                    ShowWarning();
                    sprite.sprite = aimSprite;
                }
                else
                {
                    GetComponent<AudioSource>().PlayOneShot(shotSound);
                    bunny.Kill();
                    _recoilAnim = 1f;
                }
            }
            else
            {
                if (_aiming)
                {
                    _aiming = false;
                    sprite.sprite = normalSprite;
                }
                else
                {
                    if(Random.Range(0f, 1f) < 0.5f)
                    {
                        MoveToAnotherPlace();
                    }
                    else
                    {
                        SetFlipped(!_isFlipped);
                    }
                }
            }
        }

        if (_recoilAnim > 0)
        {
            _recoilAnim -= Time.deltaTime * 4f;
            float yrot = sprite.transform.localRotation.eulerAngles.y;
            float zrot = _recoilAnim * 30f;
            sprite.transform.localRotation = Quaternion.Euler(new Vector3(0, yrot, zrot));
        }

        if(visionFeedback != null)
        {
            visionFeedback.SetActive(_aiming);
        }
    }

    void MoveToAnotherPlace()
    {
        Vector2 targetPos = Field.instance.GetRandomPosition(0.4f);
        if(Random.Range(0f, 1f) < 0.2f)
        {
            if (targetPos.y > 0)
                targetPos.y = Field.instance.bounds.max.y - sprite.bounds.size.y;
            else
                targetPos.y = Field.instance.bounds.min.y;
        }
        MoveTo(targetPos);
    }

    bool IsInField()
    {
        bool b = Field.instance.bounds.Contains(transform.position);
        //if (!b)
        //    Debug.Log(transform.position + ", " + Field.instance.bounds);
        return b;
    }

    bool _isFlipped = false;
    private void SetFlipped(bool flipped)
    {
        _isFlipped = flipped;
        if (flipped)
        {
            transform.localRotation = Quaternion.Euler(0f, 180f, 0f);
        }
        else
        {
            transform.localRotation = Quaternion.identity;
        }
    }
}

